@@ -15,11 +15,12 @@ from icontract import require
 
 from .serializers import CloudPickleSerializer, Serializer
 from .storage import CacheExpired, LocalFileStorage, Storage
+from ._logger import debug, trace
 
 
 def hash_it(*data) -> str:
     """Pickles and hashes all the data passed to it as args."""
-    result = hashlib.md5()
+    result = hashlib.md5()  # nosec B303
 
     for datum in data:
         result.update(cloudpickle.dumps(datum))
@@ -96,6 +97,7 @@ class Cache:
     cache = __call__  # Alias for backwards compatibility.
 
     @staticmethod
+    @trace
     def _get(
         key: str, serializer: Serializer, storage: Storage, deadline: dt.datetime
     ) -> Any:
@@ -103,19 +105,19 @@ class Cache:
         return serializer.loads(data)
 
     @staticmethod
+    @trace
     def _set(key: str, value: Any, serializer: Serializer, storage: Storage) -> None:
         data = serializer.dumps(value)
         storage.write(key, data)
 
     @staticmethod
-    def _get_key(
+    def _get_hash(
         fn: Callable,
         args: tuple,
         kwargs: dict,
         serializer: Serializer,
         ignore: Iterable[str],
     ) -> str:
-        """Get a cache key."""
 
         # Remove ignored arguments from the arguments tuple and kwargs dict
         arg_dict = inspect.signature(fn).bind(*args, **kwargs).arguments
@@ -185,12 +187,6 @@ class _CachedFunction:
         self.storage = storage
         self.ttl = ttl
 
-    def __repr__(self) -> str:
-        return (
-            f"<CachedFunction(cache={self.cache}, ignore={self.ignore}, "
-            "serializer={self.serializer}, storage={self.storage}, ttl={self.ttl})>"
-        )
-
     @require(
         lambda self, fn: self.ignore is None
         or all(x in inspect.signature(fn).parameters for x in self.ignore),
@@ -199,32 +195,34 @@ class _CachedFunction:
     def __call__(self, fn: Callable) -> Callable:
         """Return the correct wrapper."""
 
-        @functools.wraps(fn)
-        def _non_async_wrapper(*args, **kwargs):
-            key = self.cache._get_key(fn, args, kwargs, self.serializer, self.ignore)
-            key = self.cache._get_filename(fn, key, self.serializer)
-            try:
-                deadline = (
-                    dt.datetime.now(dt.timezone.utc) - self.ttl if self.ttl else None
-                )
-                return self.cache._get(key, self.serializer, self.storage, deadline)
-            except (FileNotFoundError, CacheExpired):
-                value = fn(*args, **kwargs)
-                self.cache._set(key, value, self.serializer, self.storage)
-                return value
+        wrapper = self._async_wrapper if is_async(fn) else self._non_async_wrapper
+        return functools.update_wrapper(functools.partial(wrapper, fn), fn)
 
-        @functools.wraps(fn)
-        async def _async_wrapper(*args, **kwargs):
-            key = self.cache._get_key(fn, args, kwargs, self.serializer, self.ignore)
-            key = self.cache._get_filename(fn, key, self.serializer)
-            try:
-                deadline = (
-                    dt.datetime.now(dt.timezone.utc) - self.ttl if self.ttl else None
-                )
-                return self.cache._get(key, self.serializer, self.storage, deadline)
-            except (FileNotFoundError, CacheExpired):
-                value = await fn(*args, **kwargs)
-                self.cache._set(key, value, self.serializer, self.storage)
-                return value
+    def _non_async_wrapper(self, fn: Callable, *args, **kwargs):
+        debug("Getting cached result for function %s", fn.__name__)
+        key = self.cache._get_hash(fn, args, kwargs, self.serializer, self.ignore)
+        key = self.cache._get_filename(fn, key, self.serializer)
+        try:
+            return self.cache._get(key, self.serializer, self.storage, self.deadline)
+        except (FileNotFoundError, CacheExpired) as exception:
+            debug("Unable to get cached result for %s: %s", fn.__name__, exception)
+            value = fn(*args, **kwargs)
+            self.cache._set(key, value, self.serializer, self.storage)
+            return value
 
-        return _async_wrapper if is_async(fn) else _non_async_wrapper
+    async def _async_wrapper(self, fn: Callable, *args, **kwargs):
+        debug("Getting cached result for function %s", fn.__name__)
+        key = self.cache._get_hash(fn, args, kwargs, self.serializer, self.ignore)
+        key = self.cache._get_filename(fn, key, self.serializer)
+        try:
+            return self.cache._get(key, self.serializer, self.storage, self.deadline)
+        except (FileNotFoundError, CacheExpired) as exception:
+            debug("Unable to get cached result for %s: %s", fn.__name__, exception)
+            value = await fn(*args, **kwargs)
+            self.cache._set(key, value, self.serializer, self.storage)
+            return value
+
+    @property
+    def deadline(self) -> dt.datetime:
+        """Return the deadline for the cache."""
+        return dt.datetime.now(dt.timezone.utc) - self.ttl if self.ttl else None
