@@ -36,14 +36,14 @@ def test_body_change(cache: Cache):
     def get_data(key):
         return key
 
-    hash1 = cache._get_key(get_data, (None,), {}, None, None)
+    hash1 = cache._get_hash(get_data, (None,), {}, None, None)
 
     @cache()
     def get_data(key):
         print("This function has been changed...")
         return key
 
-    hash2 = cache._get_key(get_data, (None,), {}, None, None)
+    hash2 = cache._get_hash(get_data, (None,), {}, None, None)
 
     assert hash1 != hash2
 
@@ -152,9 +152,11 @@ def test_hash(tmp_path):
     )
     for data1, data2 in data():
         assert data1 != data2
-        assert cache._get_key(
+        assert cache._get_hash(
             lambda x: None, (data1,), {}, CloudPickleSerializer(), None
-        ) != cache._get_key(lambda x: None, (data2,), {}, CloudPickleSerializer(), None)
+        ) != cache._get_hash(
+            lambda x: None, (data2,), {}, CloudPickleSerializer(), None
+        )
 
 
 @pytest.mark.asyncio
@@ -192,6 +194,47 @@ async def test_retrieve_with_new_cache(tmp_path):
     assert await get_data("yyy", 222) == "abc"
     assert counter == 1
 
+
+@pytest.mark.xfail(
+    reason=(
+        "There is no way to understand if the implementation of the serializer has changed"
+        " if it was created with make_serializer()"
+    )
+)
+def test_retrieve_with_new_serializer(cache):
+    counter = 0
+
+    @cache(ignore={"app", "key"})
+    def get_data(app, key):
+        nonlocal counter
+        counter += 1
+        return "abc"
+
+    DummySerializer = make_serializer(
+        "DummySerializer", "dummy", lambda x: x, lambda x: x
+    )
+    source1 = inspect.getsource(cache.serializer.dumps)
+
+    assert get_data("xxx", 111) == "abc"
+    assert counter == 1
+
+    # and now, a new serializer...
+    DummySerializer = make_serializer(
+        "DummySerializer", "dummy", lambda x: x * 2, lambda x: x * 2
+    )
+
+    source2 = inspect.getsource(cache.serializer.dumps)
+
+    assert source1 != source2  # source not the same
+
+    get_data("yyy", 222)
+
+    # should NOT be taken from cache
+    assert counter == 2
+
+
 def test_make_serializer():
-    DummySerializer = make_serializer("DummySerializer", "dummy", lambda x: x, lambda x: x)
+    DummySerializer = make_serializer(
+        "DummySerializer", "dummy", lambda x: x, lambda x: x
+    )
     assert DummySerializer.__name__ == "DummySerializer"  # important for hashing
